package com.jiuqi.monitor;

import java.util.Stack;

public abstract class AbstractProgressMonitor implements IProgressMonitor
{
    private volatile boolean canceled;
    private volatile double position;
    private Stack tasks;
    private static final double POS_DELTA = 1.0E-4;
    
    public AbstractProgressMonitor() {
        this.tasks = new Stack();
    }
    
    public void cancel() {
        this.canceled = true;
    }
    
    public void finishTask() {
        if (this.tasks.isEmpty()) {
            return;
        }
        this.tasks.pop();
        if (this.tasks.isEmpty()) {
            this.notify(this.position = 1.0);
        }
    }
    
    public double getPosition() {
        return this.position;
    }
    
    public boolean isCanceled() {
        return this.canceled;
    }
    
    public abstract void prompt(final String p0);
    
    public void startTask(final String taskName, final int stepCount) {
        ProgressTask curTask = this.tasks.isEmpty() ? null : this.tasks.peek();
        curTask = new ProgressTask(curTask, taskName, stepCount);
        this.tasks.push(curTask);
    }
    
    public void startTask(final String taskName, final int[] steps) {
        ProgressTask curTask = this.tasks.isEmpty() ? null : this.tasks.peek();
        curTask = new ProgressTask(curTask, taskName, steps);
        this.tasks.push(curTask);
    }
    
    protected abstract void notify(final double p0);
    
    public void stepIn() {
        if (this.tasks.empty()) {
            return;
        }
        final ProgressTask curTask = this.tasks.peek();
        curTask.stepIn();
        final double pos = curTask.position();
        if (pos > this.position) {
            if (pos - this.position > 1.0E-4) {
                this.notify(pos);
            }
            this.position = pos;
        }
    }
    
    public String getCurrentTask() {
        return this.tasks.isEmpty() ? null : this.tasks.peek().name();
    }
    
    public int getCurrentLevel() {
        return this.tasks.size();
    }
}
